# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.authorization_response import AuthorizationResponse

__all__ = ["ExecuteToolResponse", "Output", "OutputError", "OutputLog"]


class OutputError(BaseModel):
    can_retry: bool

    kind: Literal[
        "TOOLKIT_LOAD_FAILED",
        "TOOL_DEFINITION_BAD_DEFINITION",
        "TOOL_DEFINITION_BAD_INPUT_SCHEMA",
        "TOOL_DEFINITION_BAD_OUTPUT_SCHEMA",
        "TOOL_REQUIREMENTS_NOT_MET",
        "TOOL_RUNTIME_BAD_INPUT_VALUE",
        "TOOL_RUNTIME_BAD_OUTPUT_VALUE",
        "TOOL_RUNTIME_RETRY",
        "TOOL_RUNTIME_CONTEXT_REQUIRED",
        "TOOL_RUNTIME_FATAL",
        "UPSTREAM_RUNTIME_BAD_REQUEST",
        "UPSTREAM_RUNTIME_AUTH_ERROR",
        "UPSTREAM_RUNTIME_NOT_FOUND",
        "UPSTREAM_RUNTIME_VALIDATION_ERROR",
        "UPSTREAM_RUNTIME_RATE_LIMIT",
        "UPSTREAM_RUNTIME_SERVER_ERROR",
        "UPSTREAM_RUNTIME_UNMAPPED",
        "UNKNOWN",
    ]

    message: str

    additional_prompt_content: Optional[str] = None

    developer_message: Optional[str] = None

    extra: Optional[Dict[str, object]] = None

    retry_after_ms: Optional[int] = None

    stacktrace: Optional[str] = None

    status_code: Optional[int] = None


class OutputLog(BaseModel):
    level: str

    message: str

    subtype: Optional[str] = None


class Output(BaseModel):
    authorization: Optional[AuthorizationResponse] = None

    error: Optional[OutputError] = None

    logs: Optional[List[OutputLog]] = None

    value: Optional[object] = None


class ExecuteToolResponse(BaseModel):
    id: Optional[str] = None

    duration: Optional[float] = None

    execution_id: Optional[str] = None

    execution_type: Optional[str] = None

    finished_at: Optional[str] = None

    output: Optional[Output] = None

    run_at: Optional[str] = None

    status: Optional[str] = None

    success: Optional[bool] = None
    """
    Whether the request was successful. For immediately-executed requests, this will
    be true if the tool call succeeded. For scheduled requests, this will be true if
    the request was scheduled successfully.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["WorkerUpdateParams", "HTTP", "Mcp", "McpOauth2"]


class WorkerUpdateParams(TypedDict, total=False):
    enabled: bool

    http: HTTP

    mcp: Mcp


class HTTP(TypedDict, total=False):
    retry: int

    secret: str

    timeout: int

    uri: str


class McpOauth2(TypedDict, total=False):
    authorization_url: str

    client_id: str

    client_secret: str


class Mcp(TypedDict, total=False):
    headers: Dict[str, str]

    oauth2: McpOauth2

    retry: int

    secrets: Dict[str, str]

    timeout: int

    uri: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AuthStatusParams"]


class AuthStatusParams(TypedDict, total=False):
    id: Required[str]
    """Authorization ID"""

    wait: int
    """Timeout in seconds (max 59)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["WorkerCreateParams", "HTTP", "Mcp", "McpOauth2"]


class WorkerCreateParams(TypedDict, total=False):
    id: Required[str]

    enabled: bool

    http: HTTP

    mcp: Mcp

    type: str


class HTTP(TypedDict, total=False):
    retry: Required[int]

    secret: Required[str]

    timeout: Required[int]

    uri: Required[str]


class McpOauth2(TypedDict, total=False):
    authorization_url: str

    client_id: str

    client_secret: str

    external_id: str


class Mcp(TypedDict, total=False):
    retry: Required[int]

    timeout: Required[int]

    uri: Required[str]

    headers: Dict[str, str]

    oauth2: McpOauth2

    secrets: Dict[str, str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["UserConnectionListParams", "Provider", "User"]


class UserConnectionListParams(TypedDict, total=False):
    limit: int
    """Page size"""

    offset: int
    """Page offset"""

    provider: Provider

    user: User


class Provider(TypedDict, total=False):
    id: str
    """Provider ID"""


class User(TypedDict, total=False):
    id: str
    """User ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FormattedGetParams"]


class FormattedGetParams(TypedDict, total=False):
    format: str
    """Provider format"""

    user_id: str
    """User ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arcadepy import Arcade, AsyncArcade
from tests.utils import assert_matches_type
from arcadepy.types.admin import (
    AuthProviderResponse,
    AuthProviderListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuthProviders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.create(
            id="id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.create(
            id="id",
            description="description",
            external_id="external_id",
            oauth2={
                "client_id": "client_id",
                "authorize_request": {
                    "endpoint": "endpoint",
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "client_secret": "client_secret",
                "pkce": {
                    "code_challenge_method": "code_challenge_method",
                    "enabled": True,
                },
                "refresh_request": {
                    "endpoint": "endpoint",
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "scope_delimiter": ",",
                "token_introspection_request": {
                    "endpoint": "endpoint",
                    "triggers": {
                        "on_token_grant": True,
                        "on_token_refresh": True,
                    },
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "token_request": {
                    "endpoint": "endpoint",
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "user_info_request": {
                    "endpoint": "endpoint",
                    "triggers": {
                        "on_token_grant": True,
                        "on_token_refresh": True,
                    },
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
            },
            provider_id="provider_id",
            status="status",
            type="type",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Arcade) -> None:
        response = client.admin.auth_providers.with_raw_response.create(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Arcade) -> None:
        with client.admin.auth_providers.with_streaming_response.create(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.list()
        assert_matches_type(AuthProviderListResponse, auth_provider, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Arcade) -> None:
        response = client.admin.auth_providers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = response.parse()
        assert_matches_type(AuthProviderListResponse, auth_provider, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Arcade) -> None:
        with client.admin.auth_providers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = response.parse()
            assert_matches_type(AuthProviderListResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.delete(
            "id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Arcade) -> None:
        response = client.admin.auth_providers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Arcade) -> None:
        with client.admin.auth_providers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.admin.auth_providers.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_get(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.get(
            "id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Arcade) -> None:
        response = client.admin.auth_providers.with_raw_response.get(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Arcade) -> None:
        with client.admin.auth_providers.with_streaming_response.get(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.admin.auth_providers.with_raw_response.get(
                "",
            )

    @parametrize
    def test_method_patch(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.patch(
            path_id="id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_method_patch_with_all_params(self, client: Arcade) -> None:
        auth_provider = client.admin.auth_providers.patch(
            path_id="id",
            body_id="id",
            description="description",
            oauth2={
                "authorize_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "client_id": "client_id",
                "client_secret": "client_secret",
                "pkce": {
                    "code_challenge_method": "code_challenge_method",
                    "enabled": True,
                },
                "refresh_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "scope_delimiter": ",",
                "token_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "user_info_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                    "triggers": {
                        "on_token_grant": True,
                        "on_token_refresh": True,
                    },
                },
            },
            provider_id="provider_id",
            status="status",
            type="type",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_raw_response_patch(self, client: Arcade) -> None:
        response = client.admin.auth_providers.with_raw_response.patch(
            path_id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    def test_streaming_response_patch(self, client: Arcade) -> None:
        with client.admin.auth_providers.with_streaming_response.patch(
            path_id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_patch(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_id` but received ''"):
            client.admin.auth_providers.with_raw_response.patch(
                path_id="",
            )


class TestAsyncAuthProviders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.create(
            id="id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.create(
            id="id",
            description="description",
            external_id="external_id",
            oauth2={
                "client_id": "client_id",
                "authorize_request": {
                    "endpoint": "endpoint",
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "client_secret": "client_secret",
                "pkce": {
                    "code_challenge_method": "code_challenge_method",
                    "enabled": True,
                },
                "refresh_request": {
                    "endpoint": "endpoint",
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "scope_delimiter": ",",
                "token_introspection_request": {
                    "endpoint": "endpoint",
                    "triggers": {
                        "on_token_grant": True,
                        "on_token_refresh": True,
                    },
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "token_request": {
                    "endpoint": "endpoint",
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "user_info_request": {
                    "endpoint": "endpoint",
                    "triggers": {
                        "on_token_grant": True,
                        "on_token_refresh": True,
                    },
                    "auth_method": "auth_method",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
            },
            provider_id="provider_id",
            status="status",
            type="type",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncArcade) -> None:
        response = await async_client.admin.auth_providers.with_raw_response.create(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = await response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncArcade) -> None:
        async with async_client.admin.auth_providers.with_streaming_response.create(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = await response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.list()
        assert_matches_type(AuthProviderListResponse, auth_provider, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArcade) -> None:
        response = await async_client.admin.auth_providers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = await response.parse()
        assert_matches_type(AuthProviderListResponse, auth_provider, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArcade) -> None:
        async with async_client.admin.auth_providers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = await response.parse()
            assert_matches_type(AuthProviderListResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.delete(
            "id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncArcade) -> None:
        response = await async_client.admin.auth_providers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = await response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncArcade) -> None:
        async with async_client.admin.auth_providers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = await response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.admin.auth_providers.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_get(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.get(
            "id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncArcade) -> None:
        response = await async_client.admin.auth_providers.with_raw_response.get(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = await response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncArcade) -> None:
        async with async_client.admin.auth_providers.with_streaming_response.get(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = await response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.admin.auth_providers.with_raw_response.get(
                "",
            )

    @parametrize
    async def test_method_patch(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.patch(
            path_id="id",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_method_patch_with_all_params(self, async_client: AsyncArcade) -> None:
        auth_provider = await async_client.admin.auth_providers.patch(
            path_id="id",
            body_id="id",
            description="description",
            oauth2={
                "authorize_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "client_id": "client_id",
                "client_secret": "client_secret",
                "pkce": {
                    "code_challenge_method": "code_challenge_method",
                    "enabled": True,
                },
                "refresh_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "scope_delimiter": ",",
                "token_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                },
                "user_info_request": {
                    "auth_method": "auth_method",
                    "endpoint": "endpoint",
                    "method": "method",
                    "params": {"foo": "string"},
                    "request_content_type": "application/x-www-form-urlencoded",
                    "response_content_type": "application/x-www-form-urlencoded",
                    "response_map": {"foo": "string"},
                    "triggers": {
                        "on_token_grant": True,
                        "on_token_refresh": True,
                    },
                },
            },
            provider_id="provider_id",
            status="status",
            type="type",
        )
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_raw_response_patch(self, async_client: AsyncArcade) -> None:
        response = await async_client.admin.auth_providers.with_raw_response.patch(
            path_id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth_provider = await response.parse()
        assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

    @parametrize
    async def test_streaming_response_patch(self, async_client: AsyncArcade) -> None:
        async with async_client.admin.auth_providers.with_streaming_response.patch(
            path_id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth_provider = await response.parse()
            assert_matches_type(AuthProviderResponse, auth_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_patch(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_id` but received ''"):
            await async_client.admin.auth_providers.with_raw_response.patch(
                path_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SecretResponse", "Binding"]


class Binding(BaseModel):
    id: Optional[str] = None

    type: Optional[Literal["static", "tenant", "project", "account"]] = None


class SecretResponse(BaseModel):
    id: Optional[str] = None

    binding: Optional[Binding] = None

    created_at: Optional[str] = None

    description: Optional[str] = None

    hint: Optional[str] = None

    key: Optional[str] = None

    last_accessed_at: Optional[str] = None

    updated_at: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FormattedListParams"]


class FormattedListParams(TypedDict, total=False):
    format: str
    """Provider format"""

    limit: int
    """Number of items to return (default: 25, max: 100)"""

    offset: int
    """Offset from the start of the list (default: 0)"""

    toolkit: str
    """Toolkit name"""

    user_id: str
    """User ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ..._models import BaseModel
from ..tool_execution_attempt import ToolExecutionAttempt

__all__ = ["ScheduledGetResponse"]


class ScheduledGetResponse(BaseModel):
    id: Optional[str] = None

    attempts: Optional[List[ToolExecutionAttempt]] = None

    created_at: Optional[str] = None

    execution_status: Optional[str] = None

    execution_type: Optional[str] = None

    finished_at: Optional[str] = None

    input: Optional[Dict[str, object]] = None

    run_at: Optional[str] = None

    started_at: Optional[str] = None

    tool_name: Optional[str] = None

    toolkit_name: Optional[str] = None

    toolkit_version: Optional[str] = None

    updated_at: Optional[str] = None

    user_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .admin import (
    AdminResource,
    AsyncAdminResource,
    AdminResourceWithRawResponse,
    AsyncAdminResourceWithRawResponse,
    AdminResourceWithStreamingResponse,
    AsyncAdminResourceWithStreamingResponse,
)
from .secrets import (
    SecretsResource,
    AsyncSecretsResource,
    SecretsResourceWithRawResponse,
    AsyncSecretsResourceWithRawResponse,
    SecretsResourceWithStreamingResponse,
    AsyncSecretsResourceWithStreamingResponse,
)
from .auth_providers import (
    AuthProvidersResource,
    AsyncAuthProvidersResource,
    AuthProvidersResourceWithRawResponse,
    AsyncAuthProvidersResourceWithRawResponse,
    AuthProvidersResourceWithStreamingResponse,
    AsyncAuthProvidersResourceWithStreamingResponse,
)
from .user_connections import (
    UserConnectionsResource,
    AsyncUserConnectionsResource,
    UserConnectionsResourceWithRawResponse,
    AsyncUserConnectionsResourceWithRawResponse,
    UserConnectionsResourceWithStreamingResponse,
    AsyncUserConnectionsResourceWithStreamingResponse,
)

__all__ = [
    "UserConnectionsResource",
    "AsyncUserConnectionsResource",
    "UserConnectionsResourceWithRawResponse",
    "AsyncUserConnectionsResourceWithRawResponse",
    "UserConnectionsResourceWithStreamingResponse",
    "AsyncUserConnectionsResourceWithStreamingResponse",
    "AuthProvidersResource",
    "AsyncAuthProvidersResource",
    "AuthProvidersResourceWithRawResponse",
    "AsyncAuthProvidersResourceWithRawResponse",
    "AuthProvidersResourceWithStreamingResponse",
    "AsyncAuthProvidersResourceWithStreamingResponse",
    "SecretsResource",
    "AsyncSecretsResource",
    "SecretsResourceWithRawResponse",
    "AsyncSecretsResourceWithRawResponse",
    "SecretsResourceWithStreamingResponse",
    "AsyncSecretsResourceWithStreamingResponse",
    "AdminResource",
    "AsyncAdminResource",
    "AdminResourceWithRawResponse",
    "AsyncAdminResourceWithRawResponse",
    "AdminResourceWithStreamingResponse",
    "AsyncAdminResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .error import Error as Error
from .authorization_context import AuthorizationContext as AuthorizationContext
from .authorization_response import AuthorizationResponse as AuthorizationResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .secrets import (
    SecretsResource,
    AsyncSecretsResource,
    SecretsResourceWithRawResponse,
    AsyncSecretsResourceWithRawResponse,
    SecretsResourceWithStreamingResponse,
    AsyncSecretsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .auth_providers import (
    AuthProvidersResource,
    AsyncAuthProvidersResource,
    AuthProvidersResourceWithRawResponse,
    AsyncAuthProvidersResourceWithRawResponse,
    AuthProvidersResourceWithStreamingResponse,
    AsyncAuthProvidersResourceWithStreamingResponse,
)
from .user_connections import (
    UserConnectionsResource,
    AsyncUserConnectionsResource,
    UserConnectionsResourceWithRawResponse,
    AsyncUserConnectionsResourceWithRawResponse,
    UserConnectionsResourceWithStreamingResponse,
    AsyncUserConnectionsResourceWithStreamingResponse,
)

__all__ = ["AdminResource", "AsyncAdminResource"]


class AdminResource(SyncAPIResource):
    @cached_property
    def user_connections(self) -> UserConnectionsResource:
        return UserConnectionsResource(self._client)

    @cached_property
    def auth_providers(self) -> AuthProvidersResource:
        return AuthProvidersResource(self._client)

    @cached_property
    def secrets(self) -> SecretsResource:
        return SecretsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AdminResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AdminResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdminResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AdminResourceWithStreamingResponse(self)


class AsyncAdminResource(AsyncAPIResource):
    @cached_property
    def user_connections(self) -> AsyncUserConnectionsResource:
        return AsyncUserConnectionsResource(self._client)

    @cached_property
    def auth_providers(self) -> AsyncAuthProvidersResource:
        return AsyncAuthProvidersResource(self._client)

    @cached_property
    def secrets(self) -> AsyncSecretsResource:
        return AsyncSecretsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAdminResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAdminResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdminResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncAdminResourceWithStreamingResponse(self)


class AdminResourceWithRawResponse:
    def __init__(self, admin: AdminResource) -> None:
        self._admin = admin

    @cached_property
    def user_connections(self) -> UserConnectionsResourceWithRawResponse:
        return UserConnectionsResourceWithRawResponse(self._admin.user_connections)

    @cached_property
    def auth_providers(self) -> AuthProvidersResourceWithRawResponse:
        return AuthProvidersResourceWithRawResponse(self._admin.auth_providers)

    @cached_property
    def secrets(self) -> SecretsResourceWithRawResponse:
        return SecretsResourceWithRawResponse(self._admin.secrets)


class AsyncAdminResourceWithRawResponse:
    def __init__(self, admin: AsyncAdminResource) -> None:
        self._admin = admin

    @cached_property
    def user_connections(self) -> AsyncUserConnectionsResourceWithRawResponse:
        return AsyncUserConnectionsResourceWithRawResponse(self._admin.user_connections)

    @cached_property
    def auth_providers(self) -> AsyncAuthProvidersResourceWithRawResponse:
        return AsyncAuthProvidersResourceWithRawResponse(self._admin.auth_providers)

    @cached_property
    def secrets(self) -> AsyncSecretsResourceWithRawResponse:
        return AsyncSecretsResourceWithRawResponse(self._admin.secrets)


class AdminResourceWithStreamingResponse:
    def __init__(self, admin: AdminResource) -> None:
        self._admin = admin

    @cached_property
    def user_connections(self) -> UserConnectionsResourceWithStreamingResponse:
        return UserConnectionsResourceWithStreamingResponse(self._admin.user_connections)

    @cached_property
    def auth_providers(self) -> AuthProvidersResourceWithStreamingResponse:
        return AuthProvidersResourceWithStreamingResponse(self._admin.auth_providers)

    @cached_property
    def secrets(self) -> SecretsResourceWithStreamingResponse:
        return SecretsResourceWithStreamingResponse(self._admin.secrets)


class AsyncAdminResourceWithStreamingResponse:
    def __init__(self, admin: AsyncAdminResource) -> None:
        self._admin = admin

    @cached_property
    def user_connections(self) -> AsyncUserConnectionsResourceWithStreamingResponse:
        return AsyncUserConnectionsResourceWithStreamingResponse(self._admin.user_connections)

    @cached_property
    def auth_providers(self) -> AsyncAuthProvidersResourceWithStreamingResponse:
        return AsyncAuthProvidersResourceWithStreamingResponse(self._admin.auth_providers)

    @cached_property
    def secrets(self) -> AsyncSecretsResourceWithStreamingResponse:
        return AsyncSecretsResourceWithStreamingResponse(self._admin.secrets)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "WorkerResponse",
    "Binding",
    "HTTP",
    "HTTPSecret",
    "Mcp",
    "McpOauth2",
    "McpOauth2ClientSecret",
    "McpSecrets",
    "Requirements",
    "RequirementsAuthorization",
    "RequirementsAuthorizationOauth2",
]


class Binding(BaseModel):
    id: Optional[str] = None

    type: Optional[Literal["static", "tenant", "project", "account"]] = None


class HTTPSecret(BaseModel):
    binding: Optional[Literal["static", "tenant", "project", "account"]] = None

    editable: Optional[bool] = None

    exists: Optional[bool] = None

    hint: Optional[str] = None

    value: Optional[str] = None


class HTTP(BaseModel):
    retry: Optional[int] = None

    secret: Optional[HTTPSecret] = None

    timeout: Optional[int] = None

    uri: Optional[str] = None


class McpOauth2ClientSecret(BaseModel):
    binding: Optional[Literal["static", "tenant", "project", "account"]] = None

    editable: Optional[bool] = None

    exists: Optional[bool] = None

    hint: Optional[str] = None

    value: Optional[str] = None


class McpOauth2(BaseModel):
    authorization_url: Optional[str] = None

    client_id: Optional[str] = None

    client_secret: Optional[McpOauth2ClientSecret] = None

    redirect_uri: Optional[str] = None


class McpSecrets(BaseModel):
    binding: Optional[Literal["static", "tenant", "project", "account"]] = None

    editable: Optional[bool] = None

    exists: Optional[bool] = None

    hint: Optional[str] = None

    value: Optional[str] = None


class Mcp(BaseModel):
    headers: Optional[Dict[str, str]] = None

    oauth2: Optional[McpOauth2] = None

    retry: Optional[int] = None

    secrets: Optional[Dict[str, McpSecrets]] = None

    timeout: Optional[int] = None

    uri: Optional[str] = None


class RequirementsAuthorizationOauth2(BaseModel):
    met: Optional[bool] = None


class RequirementsAuthorization(BaseModel):
    met: Optional[bool] = None

    oauth2: Optional[RequirementsAuthorizationOauth2] = None


class Requirements(BaseModel):
    authorization: Optional[RequirementsAuthorization] = None

    met: Optional[bool] = None


class WorkerResponse(BaseModel):
    id: Optional[str] = None

    binding: Optional[Binding] = None

    enabled: Optional[bool] = None

    http: Optional[HTTP] = None

    managed: Optional[bool] = None

    mcp: Optional[Mcp] = None

    requirements: Optional[Requirements] = None

    type: Optional[Literal["http", "mcp", "unknown"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ConfirmUserResponse"]


class ConfirmUserResponse(BaseModel):
    auth_id: str

    next_uri: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .secret_response import SecretResponse

__all__ = ["SecretListResponse"]


class SecretListResponse(BaseModel):
    items: Optional[List[SecretResponse]] = None

    limit: Optional[int] = None

    offset: Optional[int] = None

    page_count: Optional[int] = None

    total_count: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.admin import auth_provider_patch_params, auth_provider_create_params
from ..._base_client import make_request_options
from ...types.admin.auth_provider_response import AuthProviderResponse
from ...types.admin.auth_provider_list_response import AuthProviderListResponse

__all__ = ["AuthProvidersResource", "AsyncAuthProvidersResource"]


class AuthProvidersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthProvidersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AuthProvidersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthProvidersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AuthProvidersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        id: str,
        description: str | Omit = omit,
        external_id: str | Omit = omit,
        oauth2: auth_provider_create_params.Oauth2 | Omit = omit,
        provider_id: str | Omit = omit,
        status: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Create a new auth provider

        Args:
          external_id: The unique external ID for the auth provider

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/admin/auth_providers",
            body=maybe_transform(
                {
                    "id": id,
                    "description": description,
                    "external_id": external_id,
                    "oauth2": oauth2,
                    "provider_id": provider_id,
                    "status": status,
                    "type": type,
                },
                auth_provider_create_params.AuthProviderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderListResponse:
        """List a page of auth providers that are available to the caller"""
        return self._get(
            "/v1/admin/auth_providers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Delete a specific auth provider

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/v1/admin/auth_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )

    def get(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Get the details of a specific auth provider

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v1/admin/auth_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )

    def patch(
        self,
        path_id: str,
        *,
        body_id: str | Omit = omit,
        description: str | Omit = omit,
        oauth2: auth_provider_patch_params.Oauth2 | Omit = omit,
        provider_id: str | Omit = omit,
        status: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Patch an existing auth provider

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_id:
            raise ValueError(f"Expected a non-empty value for `path_id` but received {path_id!r}")
        return self._patch(
            f"/v1/admin/auth_providers/{path_id}",
            body=maybe_transform(
                {
                    "body_id": body_id,
                    "description": description,
                    "oauth2": oauth2,
                    "provider_id": provider_id,
                    "status": status,
                    "type": type,
                },
                auth_provider_patch_params.AuthProviderPatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )


class AsyncAuthProvidersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthProvidersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthProvidersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthProvidersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncAuthProvidersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        id: str,
        description: str | Omit = omit,
        external_id: str | Omit = omit,
        oauth2: auth_provider_create_params.Oauth2 | Omit = omit,
        provider_id: str | Omit = omit,
        status: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Create a new auth provider

        Args:
          external_id: The unique external ID for the auth provider

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/admin/auth_providers",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "description": description,
                    "external_id": external_id,
                    "oauth2": oauth2,
                    "provider_id": provider_id,
                    "status": status,
                    "type": type,
                },
                auth_provider_create_params.AuthProviderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderListResponse:
        """List a page of auth providers that are available to the caller"""
        return await self._get(
            "/v1/admin/auth_providers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Delete a specific auth provider

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/v1/admin/auth_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )

    async def get(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Get the details of a specific auth provider

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v1/admin/auth_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )

    async def patch(
        self,
        path_id: str,
        *,
        body_id: str | Omit = omit,
        description: str | Omit = omit,
        oauth2: auth_provider_patch_params.Oauth2 | Omit = omit,
        provider_id: str | Omit = omit,
        status: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthProviderResponse:
        """
        Patch an existing auth provider

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_id:
            raise ValueError(f"Expected a non-empty value for `path_id` but received {path_id!r}")
        return await self._patch(
            f"/v1/admin/auth_providers/{path_id}",
            body=await async_maybe_transform(
                {
                    "body_id": body_id,
                    "description": description,
                    "oauth2": oauth2,
                    "provider_id": provider_id,
                    "status": status,
                    "type": type,
                },
                auth_provider_patch_params.AuthProviderPatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthProviderResponse,
        )


class AuthProvidersResourceWithRawResponse:
    def __init__(self, auth_providers: AuthProvidersResource) -> None:
        self._auth_providers = auth_providers

        self.create = to_raw_response_wrapper(
            auth_providers.create,
        )
        self.list = to_raw_response_wrapper(
            auth_providers.list,
        )
        self.delete = to_raw_response_wrapper(
            auth_providers.delete,
        )
        self.get = to_raw_response_wrapper(
            auth_providers.get,
        )
        self.patch = to_raw_response_wrapper(
            auth_providers.patch,
        )


class AsyncAuthProvidersResourceWithRawResponse:
    def __init__(self, auth_providers: AsyncAuthProvidersResource) -> None:
        self._auth_providers = auth_providers

        self.create = async_to_raw_response_wrapper(
            auth_providers.create,
        )
        self.list = async_to_raw_response_wrapper(
            auth_providers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            auth_providers.delete,
        )
        self.get = async_to_raw_response_wrapper(
            auth_providers.get,
        )
        self.patch = async_to_raw_response_wrapper(
            auth_providers.patch,
        )


class AuthProvidersResourceWithStreamingResponse:
    def __init__(self, auth_providers: AuthProvidersResource) -> None:
        self._auth_providers = auth_providers

        self.create = to_streamed_response_wrapper(
            auth_providers.create,
        )
        self.list = to_streamed_response_wrapper(
            auth_providers.list,
        )
        self.delete = to_streamed_response_wrapper(
            auth_providers.delete,
        )
        self.get = to_streamed_response_wrapper(
            auth_providers.get,
        )
        self.patch = to_streamed_response_wrapper(
            auth_providers.patch,
        )


class AsyncAuthProvidersResourceWithStreamingResponse:
    def __init__(self, auth_providers: AsyncAuthProvidersResource) -> None:
        self._auth_providers = auth_providers

        self.create = async_to_streamed_response_wrapper(
            auth_providers.create,
        )
        self.list = async_to_streamed_response_wrapper(
            auth_providers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            auth_providers.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            auth_providers.get,
        )
        self.patch = async_to_streamed_response_wrapper(
            auth_providers.patch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "AuthProviderResponse",
    "Binding",
    "Oauth2",
    "Oauth2AuthorizeRequest",
    "Oauth2ClientSecret",
    "Oauth2Pkce",
    "Oauth2RefreshRequest",
    "Oauth2TokenIntrospectionRequest",
    "Oauth2TokenIntrospectionRequestTriggers",
    "Oauth2TokenRequest",
    "Oauth2UserInfoRequest",
    "Oauth2UserInfoRequestTriggers",
]


class Binding(BaseModel):
    id: Optional[str] = None

    type: Optional[Literal["static", "tenant", "project", "account"]] = None


class Oauth2AuthorizeRequest(BaseModel):
    auth_method: Optional[str] = None

    endpoint: Optional[str] = None

    expiration_format: Optional[str] = None

    method: Optional[str] = None

    params: Optional[Dict[str, str]] = None

    request_content_type: Optional[str] = None

    response_content_type: Optional[str] = None

    response_map: Optional[Dict[str, str]] = None


class Oauth2ClientSecret(BaseModel):
    binding: Optional[Literal["static", "tenant", "project", "account"]] = None

    editable: Optional[bool] = None

    exists: Optional[bool] = None

    hint: Optional[str] = None

    value: Optional[str] = None


class Oauth2Pkce(BaseModel):
    code_challenge_method: Optional[str] = None

    enabled: Optional[bool] = None


class Oauth2RefreshRequest(BaseModel):
    auth_method: Optional[str] = None

    endpoint: Optional[str] = None

    expiration_format: Optional[str] = None

    method: Optional[str] = None

    params: Optional[Dict[str, str]] = None

    request_content_type: Optional[str] = None

    response_content_type: Optional[str] = None

    response_map: Optional[Dict[str, str]] = None


class Oauth2TokenIntrospectionRequestTriggers(BaseModel):
    on_token_grant: Optional[bool] = None

    on_token_refresh: Optional[bool] = None


class Oauth2TokenIntrospectionRequest(BaseModel):
    auth_method: Optional[str] = None

    enabled: Optional[bool] = None

    endpoint: Optional[str] = None

    expiration_format: Optional[str] = None

    method: Optional[str] = None

    params: Optional[Dict[str, str]] = None

    request_content_type: Optional[str] = None

    response_content_type: Optional[str] = None

    response_map: Optional[Dict[str, str]] = None

    triggers: Optional[Oauth2TokenIntrospectionRequestTriggers] = None


class Oauth2TokenRequest(BaseModel):
    auth_method: Optional[str] = None

    endpoint: Optional[str] = None

    expiration_format: Optional[str] = None

    method: Optional[str] = None

    params: Optional[Dict[str, str]] = None

    request_content_type: Optional[str] = None

    response_content_type: Optional[str] = None

    response_map: Optional[Dict[str, str]] = None


class Oauth2UserInfoRequestTriggers(BaseModel):
    on_token_grant: Optional[bool] = None

    on_token_refresh: Optional[bool] = None


class Oauth2UserInfoRequest(BaseModel):
    auth_method: Optional[str] = None

    endpoint: Optional[str] = None

    expiration_format: Optional[str] = None

    method: Optional[str] = None

    params: Optional[Dict[str, str]] = None

    request_content_type: Optional[str] = None

    response_content_type: Optional[str] = None

    response_map: Optional[Dict[str, str]] = None

    triggers: Optional[Oauth2UserInfoRequestTriggers] = None


class Oauth2(BaseModel):
    authorize_request: Optional[Oauth2AuthorizeRequest] = None

    client_id: Optional[str] = None

    client_secret: Optional[Oauth2ClientSecret] = None

    pkce: Optional[Oauth2Pkce] = None

    redirect_uri: Optional[str] = None
    """The redirect URI required for this provider."""

    refresh_request: Optional[Oauth2RefreshRequest] = None

    scope_delimiter: Optional[str] = None

    token_introspection_request: Optional[Oauth2TokenIntrospectionRequest] = None

    token_request: Optional[Oauth2TokenRequest] = None

    user_info_request: Optional[Oauth2UserInfoRequest] = None


class AuthProviderResponse(BaseModel):
    id: Optional[str] = None

    binding: Optional[Binding] = None

    created_at: Optional[str] = None

    description: Optional[str] = None

    oauth2: Optional[Oauth2] = None

    provider_id: Optional[str] = None

    status: Optional[str] = None

    type: Optional[str] = None

    updated_at: Optional[str] = None
